
/*
 * DEFS.H
 */

#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <sys/file.h>

#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/*#include <bstring.h>*/
#include <stdarg.h>
#include <stddef.h>
#include <math.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>

#include "config.h"

/*#include <reldb/reldb.h>*/

#define Prototype extern

#define arysize(ary)	(sizeof(ary)/sizeof((ary)[0]))

typedef struct Header {
    const char	*he_Base;
    int		he_Bytes;
} Header;

typedef struct ListVar {
    struct ListVar *lv_Next;
    char	*lv_Name;
    char	*lv_Data;
} ListVar;

typedef struct MailStats {
    int		st_DigestMsgs;
    int		st_DigestSize;
    int		st_DigestVol;
    int		st_DigestNo;
    long	st_DigestPoll;
    long	st_DigestNext;	/* for queue runs			*/

    struct MailStats *st_Next;
    char	*st_ListName;
    FILE	*st_Fi;
    int		st_Refs;
} MailStats;

typedef struct MailList {
    struct MailList *ml_Next;
    char	*ml_ListName;
    ListVar	*ml_First;
    int		ml_DirtyVar;
    int		ml_Refs;
    int		ml_ListFileLocked;
    FILE	*ml_Fi;
    MailStats	ml_Stats;
} MailList;

typedef struct Msg {
    char	*ms_ListName;	/* name of list				*/
    MailList	*ms_List;	/* associated mailing list		*/
    char	*ms_SubSystem;	/* name of subsystem			*/
    char	*ms_QueueFile;	/* name of queue file			*/
    char	*ms_Id;		/* assigned id (queuefile.random)	*/
    long	ms_IdOff;	/* offset of 'I' control line in file	*/
    const char	*ms_Map;	/* message base				*/
    int		ms_Size;
    int		ms_NumHead;	/* number of headers			*/
    char	ms_DeleteFlag;	/* delete on dispose			*/
    char	ms_ErrorFlag;	/* error flag				*/
    char	ms_State;	/* message state			*/
    char	ms_BounceFlag;	/* please bounce this message		*/
    char	ms_IncludeCompleteMessage;	/* entire msg on response */
    char	ms_ExecScanOnly;/* prescan of commands			*/
    char	ms_ExecScanFlags;/* results of scan			*/
    int		ms_Fd;
    int		ms_QFO;
    const char	*ms_ErrorMsg;
    Header	*ms_Headers;	/* array of headers	*/
    Header	ms_Body;	/* message body		*/
    char	*ms_ReturnBuf;
    int		ms_RBLen;
    struct Msg  *ms_OriginalMsg;
} Msg;

#define EXSF_SUBSCRIBE		0x01
#define EXSF_UNSUBSCRIBE	0x02
#define EXSF_GENERAL		0x04
#define EXSF_MODERATOR		0x08

#define FROM_UNKNOWN		0
#define FROM_MODERATOR		1
#define FROM_MODERATOR_AUTH	2

#define BOUNCE_RETURN		0x01
#define BOUNCE_AUTHENTICATE	0x02
#define BOUNCE_AUTHMODERATOR	0x04
#define BOUNCE_CCMODERATOR	0x08	/* CC response to moderator	*/
#define BOUNCE_FWD_MODERATOR	0x10	/* forward email to moderator 	*/
#define BOUNCE_CCERRORS		0x20	/* CC to errorsCC		*/

#define MSSTATE_PARTIAL		'P'
#define MSSTATE_ACTIVE		'A'
#define MSSTATE_AUTHENTICATED1	'X'
#define MSSTATE_AUTHENTICATED2	'Y'
#define MSSTATE_AUTHSUBSCRIBE	'G'

#define POLICY_NONE		'N'
#define POLICY_AUTHENTICATE	'A'
#define POLICY_REQUIRESUB	'S'

#define MODERATION_NONE		'N'
#define MODERATION_CC		'C'
#define MODERATION_FULL		'Y'

#define CHECK_EXEC		1
#define CHECK_BOUNCE		2
#define CHECK_AUTHENTICATE	3
#define CHECK_SENDMODERATOR	4

#include "bestserv-protos.h"

