/*
 * FLAME.C
 *
 * Flame war filter - bounce by subject to terminate a thread that has
 * turned into a flame war.
 */

#include "defs.h"

Prototype int CheckFlameMail(Msg *msg);

/*
 * Check whether a piece of mail has been flame-banned based 
 * on subject line.
 *
 * Returns 0 if not banned, -1 if banned.
 */
int
CheckFlameMail(Msg *msg)
{
    char buf[256];
    const char *p;
    char *subj;
    FILE *fi;
    int len;
    int ret = 0;

    p = findHeader(msg, "Subject:", 0, &len, "");
    subj = dupnstr(p, len);

    fi = OpenListFile(msg->ms_List->ml_ListName, "flamed", 0, LOCK_SH);
    if (fi != NULL) {
	while (fgets(buf, sizeof(buf), fi) != NULL) {
	    len = strlen(buf);
	    if (len == 0 || buf[len-1] != '\n')
		continue;
	    buf[len-1] = 0;
	    if (WildCaseCmp(buf, subj) == 0) {
		ret = -1;
		break;
	    }
	}
	fclose(fi);
    }
    free(subj);
    return(ret);
}

