
/*
 * LIST.C
 */

#include "defs.h"

Prototype MailList *GetMailList(const char *listName, int nb);
Prototype const char *GetListVar(MailList *list, const char *key, int n, const char *def);
Prototype void SetListVar(MailList *list, const char *key, int n, const char *val);
Prototype int GetListVal(MailList *list, const char *key, int n, int def);
Prototype FILE *OpenListFile(const char *listName, const char *fileName, int fdmodes, int lock);
Prototype void RenameListFile(const char *listName, const char *file1, const char *file2);
Prototype void RemoveListFile(const char *listName, const char *file);
Prototype int FindListAddress(MailList *list, const char *email, int len);
Prototype int GetListFile(FILE *fi, char *buf, int bufSize, char **pflags, char **pefull);
Prototype int SetListFlags(MailList *list, const char *email, int len, char *flags);
Prototype void CloseMailList(MailList *list);
Prototype void FlushMailList(MailList *list);
Prototype int DumpLines(const char *varName, MailList *list, Msg *msg, FILE *fo, int fd);

MailList *MLBase;

MailList *
GetMailList(const char *listName, int nb)
{
    MailList *ml = NULL;

    if (listName != NULL) {
	for (ml = MLBase; ml; ml = ml->ml_Next) {
	    if (strcmp(listName, ml->ml_ListName) == 0)
		break;
	}
    }

    if (ml == NULL && listName != NULL && strlen(listName) < 64) {
	FILE *fi;

	if ((fi = OpenListFile(listName, "config", 0, LOCK_EX|nb)) != NULL) {
	    char buf[1024];
	    int state = 0;
	    ListVar **plv;

	    ml = zalloc(sizeof(MailList) + strlen(listName) + 1);
	    ml->ml_ListName = (char *)(ml + 1);
	    ml->ml_Fi = fi;
	    ml->ml_Next = MLBase;
	    MLBase = ml;
	    strcpy(ml->ml_ListName, listName);

	    plv = &ml->ml_First;

	    while (fgets(buf, sizeof(buf), fi) != NULL) {
		char *ptr;

		if ((ptr = strchr(buf, '#')) != NULL)
		    strcpy(ptr, "\n");
		if (buf[0] == '\n' || buf[0] == ' ' || buf[0] == '\t')
		    continue;
		if (state == 0 && strncmp(buf, "BEGIN", 5) == 0)
		    state = 1;
		if (state == 1 && strncmp(buf, "END", 3) == 0)
		    break;
		if (state == 1) {
		    char *key = strtok(buf, " \t\n\r");
		    char *arg = strtok(NULL, "\n\r");

		    if (arg) {
			int l;

			while (*arg == ' ' || *arg == '\t')
			    ++arg;
			for (l = strlen(arg) - 1; l >= 0; --l) {
			    if (arg[l] != ' ' && arg[l] != '\t')
				break;
			}
			++l;
			arg[l] = 0;
		    } else {
			arg = "";
		    }

		    if (key) {
			ListVar *lv = zalloc(sizeof(ListVar) + strlen(key) + 1);
			lv->lv_Name = (char *)(lv + 1);
			strcpy(lv->lv_Name, key);
			lv->lv_Data = dupstr(arg);
			*plv = lv;
			plv = &lv->lv_Next;
		    }
		}
	    }
	    fseek(fi, 0L, 0);
	}
    }
    if (ml) {
	++ml->ml_Refs;
    }
    return(ml);
}

void
CloseMailList(MailList *list)
{
    if (list) {
	if (--list->ml_Refs == 0) {
	    MailList **plist;

	    for (plist = &MLBase; 
		*plist != NULL && *plist != list; 
		plist = &(*plist)->ml_Next
	    ) {
		;
	    }
	    if (*plist == NULL)
		fatal(0, "CloseMailList: consistancy error");
#ifdef NOTDEF
	    if (list->ml_DirtyVar == 1)
		FlushMailList(list);
#endif
	    *plist = list->ml_Next;
	    if (list->ml_Fi) {
		fclose(list->ml_Fi);
		list->ml_Fi = NULL;
	    }
	    zfree(list, sizeof(MailList) + strlen(list->ml_ListName) + 1);
	}
    }
}

#ifdef NOTDEF
		
void
FlushMailList(MailList *list)
{
    FILE *fo;
    char path1[256];
    char path2[256];

    snprintf(path1, sizeof(path1), "%s/lists/%s/config", BaseDir, list->ml_ListName);
    snprintf(path2, sizeof(path2), "%s/lists/%s/config.tmp", BaseDir, list->ml_ListName);

    if ((fo = fopen(path2, "w")) != NULL) {
	ListVar *lv;

	for (lv = list->ml_First; lv; lv = lv->lv_Next) {
	    if (lv->lv_Data && lv->lv_Data[0])
		fprintf(fo, "%s\t%s\n", lv->lv_Name, lv->lv_Data);
	    else
		fprintf(fo, "%s\n", lv->lv_Name);
	}
	fflush(fo);
	if (ferror(fo) == 0) {
	    rename(path2, path1);
	} else {
	    remove(path2);
	    /* LOG ERROR */
	}
	fclose(fo);
    }
}

#endif

const char *
GetListVar(MailList *list, const char *key, int n, const char *def)
{
    ListVar *lv;
    char nbuf[32];
    int keyLen = strlen(key);

    if (n)
	snprintf(nbuf, sizeof(nbuf), "%d", n);

    if (list) {
	for (lv = list->ml_First; lv; lv = lv->lv_Next) {
	    if (n == 0 && strcmp(lv->lv_Name, key) == 0) {
		def = lv->lv_Data;
		break;
	    }
	    if (n != 0 && strncmp(lv->lv_Name, key, keyLen) == 0) {
		if (strcmp(lv->lv_Name + keyLen, nbuf) == 0) {
		    def = lv->lv_Data;
		    break;
		}
	    }
	}
    }
    return(def);
}

#ifdef NOTDEF

void 
SetListVar(MailList *list, const char *key, int n, const char *val)
{
    ListVar *lv;
    ListVar **plv;
    char nbuf[32];
    int keyLen = strlen(key);

    if (n) 
	snprintf(nbuf, sizeof(nbuf), "%d", n);
    
    if (list) {
	for (plv = &list->ml_First; (lv = *plv) != NULL; plv = &lv->lv_Next) {
	    if (n == 0 && strcmp(lv->lv_Name, key) == 0)
		break;
	    if (n != 0 && strncmp(lv->lv_Name, key, keyLen) == 0) {
		if (strcmp(lv->lv_Name + keyLen, nbuf) == 0) {
		    break;
		}
	    }
	}
	if (lv == NULL && val != NULL) {
	    lv = zalloc(sizeof(ListVar) + strlen(key) + ((n) ? strlen(nbuf) : 0) + 1);
	    lv->lv_Name = (char *)(lv + 1);
	    strcpy(lv->lv_Name, key);
	    if (n)
		strcat(lv->lv_Name, nbuf);
	    *plv = lv;
	}
	if (lv != NULL) {
	    if (val == NULL) {
		*plv = lv->lv_Next;
		if (lv->lv_Data)
		    freestr(&lv->lv_Data);
		zfree(lv, sizeof(ListVar) + strlen(lv->lv_Name) + 1);
		list->ml_DirtyVar = 1;
	    } else if (lv->lv_Data == NULL || strcmp(val, lv->lv_Data) != 0) {
		freestr(&lv->lv_Data);
		lv->lv_Data = dupstr(val);
		list->ml_DirtyVar = 1;
	    }
	}
    }
}

#endif

int
GetListVal(MailList *list, const char *key, int n, int def)
{
    const char *res = GetListVar(list, key, n, "");

    if (res[0]) {
	char *ptr;

	def = strtol(res, &ptr, 0);
	switch(*ptr) {
	case 'k':
	    def *= 1024;
	    break;
	case 'm':
	    def *= 1024 * 1024;
	    break;
	case 'h':
	    def *= 60 * 60;
	    break;
	case 'd':
	    def *= 60 * 60 * 24;
	    break;
	}
    }
    return(def);
}

FILE *
OpenListFile(const char *listName, const char *fileName, int fdmodes, int lock)
{
    char path[256];
    FILE *fi = NULL;
    int fd;

    snprintf(path, sizeof(path), "%s/lists/%s/%s", BaseDir, listName, fileName);
loop:
    if ((fd = open(path, O_RDWR | fdmodes, 0600)) >= 0) {
	/*
	 * It might get ripped out from under us, reopen
	 * if it does.
	 */
	int r = 0;

	if (lock >= 0 && (r = flock(fd, lock)) >= 0) {
	    struct stat st;
	    if (fstat(fd, &st) == 0 && st.st_nlink == 0) {
		close(fd);
		goto loop;
	    }
	}
	if (r < 0) {
	    close(fd);
	} else {
	    fi = fdopen(fd, "r+");
	}
    }
    return(fi);
}

void 
RenameListFile(const char *listName, const char *file1, const char *file2)
{
    char path1[256];
    char path2[256];

    snprintf(path1, sizeof(path1), "%s/lists/%s/%s", BaseDir, listName, file1);
    snprintf(path2, sizeof(path2), "%s/lists/%s/%s", BaseDir, listName, file2);
    rename(path1, path2);
}

void 
RemoveListFile(const char *listName, const char *file)
{
    char path[256];

    snprintf(path, sizeof(path), "%s/lists/%s/%s", BaseDir, listName, file);
    remove(path);
}

int
GetListFile(FILE *fi, char *buf, int bufSize, char **pflags, char **pefull)
{
    int r = -1;

    while (fgets(buf, bufSize, fi) != NULL) {
	if (buf[0] == '#')
	    continue;
	if (buf[0] == '\n')
	    continue;
	if ((*pflags = strtok(buf, " \t\n\r")) != NULL) {
	    if (strchr(*pflags, '-') != NULL)
		continue;
	    if (strchr(*pflags, '0') != NULL)
		continue;
	    if ((*pefull = strtok(NULL, "\n\r")) != NULL) {
		r = 0;
		break;
	    }
	}
    }
    return(r);
}


int
FindListAddress(MailList *list, const char *email, int len)
{
    FILE *fi;
    int r = -1;

    if ((fi = OpenListFile(list->ml_ListName, "list", 0, LOCK_SH)) != NULL) {
	char buf[256];

	while (fgets(buf, sizeof(buf), fi) != NULL) {
	    const char *flags = strtok(buf, " \t\r\n");
	    const char *sem = strtok(NULL, "\r\n");
	    int slen;

	    if (flags == NULL || sem == NULL)
		continue;
	    if (strchr(flags, '+') == NULL)
		continue;
	    sem = parseEMailFromAddress(sem, strlen(sem), &slen);
	    if (len == slen && strncasecmp(email, sem, len) == 0) {
		r = 0;
		break;
	    }
	}
	fclose(fi);
    }
    return(r);
}

/*
 * SetListFlags() - used to subscribe or unsubscribe someone to the list
 */

int
SetListFlags(MailList *list, const char *email, int len, char *flags)
{
    FILE *fi;
    int r = -1;
    int lckFlags = (list->ml_ListFileLocked) ? -1 : LOCK_EX;
    char fbuf[9];

    {
	int i;

	snprintf(fbuf, sizeof(fbuf), "%-8.8s", flags);
	for (i = 0; i < 8; ++i) {
	    if (fbuf[i] == ' ')
		fbuf[i] = '.';
	}
    }

    if ((fi = OpenListFile(list->ml_ListName, "list", O_CREAT, lckFlags)) != NULL) {
	char buf[256];
	long newPos = 0;

	while (fgets(buf, sizeof(buf), fi) != NULL) {
	    const char *iflags;
	    const char *sem;
	    int slen;
	    int pos = newPos;

	    newPos = ftell(fi);

	    /*
	     * ignore comments or badly formed entries
	     */

	    if (buf[0] == '#')
		continue;

	    if ((iflags = strtok(buf, " \t\r\n")) == NULL)
		continue;

	    if ((sem = strtok(NULL, "\r\n")) == NULL)
		continue;
	    
	    /*
	     * ignore inactive (deleted) entries
	     */

	    if (strchr(iflags, '-') != NULL)
		continue;

	    /*
	     * Locate email address, update flags
	     */

	    sem = parseEMailFromAddress(sem, strlen(sem), &slen);
	    if (len == slen && strncasecmp(email, sem, len) == 0) {
		fseek(fi, pos, 0);
		fwrite(fbuf, 8, 1, fi);
		fflush(fi);
		/* ignore any error */
		r = 0;
		break;
	    }
	}
	/*
	 * Append new entry if subscription, return error if unsubscribe
	 * and list address could not be found
	 */
	if (r == -1 && strchr(flags, '+') != NULL) {
	    fprintf(fi, "%s %*.*s\n", fbuf, len, len, email);
	    fflush(fi);

	    /*
	     * If we are unable to append the new entry, remove it from
	     * the file.
	     */
	    if (ferror(fi)) {
		ftruncate(fileno(fi), newPos);
		r = -1;
	    } else {
		r = 0;
	    }
	}
	fclose(fi);
    }
    return(r);
}

int
DumpLines(const char *varName, MailList *list, Msg *msg, FILE *fo, int fd)
{
    int n;
    int r = 0;
    int blank = -1;
    const char *str;

    for (
	n = 1;
	(str = GetListVar(list, varName, n, NULL)) != NULL;
	++n
    ) {
	if (*str == 0) {
	    if (blank)
		continue;
	    blank = 1;
	} else {
	    if (blank < 0) {
		/*
		 * blank line before first header line
		 */
		if (fo)
		    fprintf(fo, "\n");
		else if (fd >= 0)
		    cprintf(fd, "\n");
		else if (msg)
		    msgprintf(msg, "\n");
	    }
	    blank = 0;
	}
	if (fo)
	    fprintf(fo, "%s\n", str);
	else if (fd >= 0)
	    cprintf(fd, "%s\n", str);
	else if (msg)
	    msgprintf(msg, "%s\n", str);
	++r;
    }
    /*
     * blank line after last header line
     */
    if (blank == 0) {
	if (fo)
	    fprintf(fo, "\n");
	else if (fd >= 0)
	    cprintf(fd, "\n");
	else if (msg)
	    msgprintf(msg, "\n");
    }

    return(r);
}

