
/*
 * MAIN.C	- BESTSERV MAIN MODULE
 */

#include "defs.h"

int
main(int ac, char **av)
{
    int i;
    char *listName = NULL;
    char *subSystem = NULL;
    int commandOpt = 0;
    int queueOpt = 0;
    int pruneOpt = 0;
    int digestOpt = 0;
    int r = 0;
    Msg *msg = NULL;
    const char *errMsg = NULL;

    gethostname(HostName, sizeof(HostName) - 1);

    for (i = 1; i < ac; ++i) {
	char *ptr = av[i];

	if (*ptr != '-') {
	    fprintf(stderr, "Illegal argument to %s: %s\n",
		av[0],
		ptr
	    );
	    exit(1);
	}
	ptr += 2;

	switch(ptr[-1]) {
	case 'v':
	    VerboseOpt = 1;
	    break;
	case 'r':	/* relate to list L	*/
	    listName = (*ptr) ? ptr : av[++i];
	    break;
	case 'c':	/* command input	*/
	    commandOpt = 1;
	    subSystem = "command";
	    break;
	case 'q':	/* run-queue		*/
	    queueOpt = 1;
	    break;
	case 'd':	/* digest related	*/
	    digestOpt = 1;
	    break;
	case 'e':	/* run error prune, disable	*/
	    pruneOpt = 1;
	    if (*ptr == 'e')	/* weekly spinup */
		pruneOpt = 2;
	    break;
	case 'E':	/* run error prune, reenable	*/
	    pruneOpt = -1;
	    break;
	case 's':	/* subsystem mail (-owner, -this, -that, ...)	*/
	    subSystem = (*ptr) ? ptr : av[++i];
	    break;
	default:
	    fprintf(stderr, "Warning: unsupported option: %s, ignored\n", 
		ptr - 2
	    );
	    break;
	}
    }
    if (i > ac) {
	fprintf(stderr, "Expected argument to option\n");
	exit(1);
    }

    {
	FILE *fi;
	char buf[256];

	snprintf(buf, sizeof(buf), "%s/hostname", BaseDir);
	if ((fi = fopen(buf, "r")) != NULL) {
	    int l;

	    fgets(HostName, sizeof(HostName), fi);
	    if ((l = strlen(HostName)) > 0 && HostName[l-1] == '\n')
		HostName[--l] = 0;
	    fclose(fi);
	}
    }
    if (VerboseOpt)
	fprintf(stderr, "HostName: %s\n", HostName);

    if (stringOk(listName, &errMsg) < 0)
	fatal(0, "Bad listname: %s", errMsg);

    if (stringOk(subSystem, &errMsg) < 0)
	fatal(0, "Bad subsystem: %s", errMsg);

    /*
     * Collect and queue message if not running
     * a batch.
     */

    if (queueOpt == 0 && pruneOpt == 0) {
	if (subSystem == NULL)
	    fatal(0, "subsystem not specified");
	msg = collectMsg(0, NULL, listName, subSystem, 0);
	if (msg == NULL) {
	    exit(99);	/* re-queue XXX	*/
	}
    }

    /*
     * Issue Command.
     *
     * We ignore messages transmitted by us or by another list XXX
     */

    if (commandOpt && msg) {
	RunMsg(msg);
    } else if (listName && msg) {
	RunMsg(msg);
    } else if (!queueOpt && !pruneOpt) {
	fprintf(stderr, "Empty Command\n");
	r = 1;
    }
    if (pruneOpt)
	RunPrune(listName, pruneOpt);

    if (queueOpt)
	RunQueue(listName);


    /*
     * Dispose of work
     */

    if (msg)
	disposeMsg(msg);

    return(r);
}

