
/*
 * MSTAT.C
 */

#include "defs.h"
#include <netinet/in.h>

Prototype MailStats *ReadMailStats(const char *listName);
Prototype void WriteMailStats(MailStats *stats);
Prototype void RelsMailStats(MailStats *stats);

MailStats *STBase;

#define CONV_HTONL(var)	var = htonl(var)
#define CONV_NTOHL(var)	var = ntohl(var)

MailStats * 
ReadMailStats(const char *listName)
{
    MailStats *stats;

    for (stats = STBase; stats; stats = stats->st_Next) {
	if (strcmp(listName, stats->st_ListName) == 0)
	    break;
    }
    if (stats == NULL) {
	FILE *fi;

	stats = zalloc(sizeof(MailStats) + strlen(listName) + 1);
	stats->st_Next = STBase;
	stats->st_ListName = (char *)(stats + 1);
	strcpy(stats->st_ListName, listName);
	STBase = stats;
	
	if ((fi = OpenListFile(listName, "stats", O_CREAT, LOCK_EX)) != NULL) {
	    fread(stats, 1, offsetof(MailStats, st_Next), fi);
	    CONV_NTOHL(stats->st_DigestMsgs);
	    CONV_NTOHL(stats->st_DigestSize);
	    CONV_NTOHL(stats->st_DigestVol);
	    CONV_NTOHL(stats->st_DigestNo);
	    CONV_NTOHL(stats->st_DigestPoll);
	    CONV_NTOHL(stats->st_DigestNext);
	}
	stats->st_Fi = fi;
    }
    if (stats->st_DigestVol == 0)
	stats->st_DigestVol = 1;
    if (stats->st_DigestNo == 0)
	stats->st_DigestNo = 1;
    if (stats->st_DigestPoll == 0)
	stats->st_DigestPoll = (long)time(NULL);
    ++stats->st_Refs;
    return(stats);
}

void 
WriteMailStats(MailStats *stats)
{
    if (stats->st_Fi) {
	MailStats copy = *stats;

	fseek(stats->st_Fi, 0L, 0);
	CONV_HTONL(copy.st_DigestMsgs);
	CONV_HTONL(copy.st_DigestSize);
	CONV_HTONL(copy.st_DigestVol);
	CONV_HTONL(copy.st_DigestNo);
	CONV_HTONL(copy.st_DigestPoll);
	CONV_HTONL(copy.st_DigestNext);
	fwrite(&copy, offsetof(MailStats, st_Next), 1, stats->st_Fi);
	fflush(stats->st_Fi);
    }
    RelsMailStats(stats);
}

void
RelsMailStats(MailStats *stats)
{
    if (--stats->st_Refs == 0) {
	int l = strlen(stats->st_ListName) + 1;
	MailStats **pst;

	for (pst = &STBase; *pst != stats && *pst != NULL; pst = &(*pst)->st_Next) {
	    ;
	}
	if (*pst == stats)
	    *pst = stats->st_Next;
	fclose(stats->st_Fi);
	bzero(stats, sizeof(*stats));
	zfree(stats, sizeof(MailStats) + l);
    }
}

