
/*
 * ORDERLIST.C
 *
 * - remove deleted entries XXX
 * - sort by domain
 */

#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/file.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>

#include "config.h"

int OpenAndLock(const char *fileName);
void SortList(FILE *fi, FILE *fo, FILE *fobak);
int emCmp(const void *s1, const void *s2);

int ForceOpt = 0;

int
main(int ac, char **av)
{
    DIR *dir;

    /* chdir to the lists directory */
    {
	char path[MAXPATHLEN];
	snprintf(path, sizeof(path), "%s/lists", BESTSERV_BASE_DIR);
	if (chdir(path) != 0) {
	    printf("Unable to chdir(\"%s\")\n", path);
	    exit(1);
	}
    }

    /*
     * scan lists directory for lists to process
     */

    if ((dir = opendir(".")) != NULL) {
	struct dirent *den;

	while ((den = readdir(dir)) != NULL) {
	    char path1[1024];
	    char path2[1024];
	    char path3[1024];
	    int fd;

	    if (strcmp(den->d_name, ".") == 0)
		continue;
	    if (strcmp(den->d_name, "..") == 0)
		continue;
	    snprintf(path1, sizeof(path1), "%s/list.ordertmp", den->d_name);
	    snprintf(path2, sizeof(path2), "%s/list", den->d_name);
	    snprintf(path3, sizeof(path3), "%s/list.bak", den->d_name);
	    if ((fd = OpenAndLock(path2)) >= 0) {
		FILE *fo = NULL;
		FILE *fo2 = NULL;
		FILE *fi = fdopen(fd, "r+");
		
		if ((fo = fopen(path1, "w")) != NULL && (fo2 = fopen(path3, "w")) != NULL) {
		    /*
		     * Sort the list by reverse-ordered-domain
		     *
		     * NOTE: cannot simply link() to backup file because st_nlink will
		     * then not wind up 0, and other processes waiting on the descriptor
		     * lock will modify the wrong file.
		     */

		    SortList(fi, fo, fo2);

		    /*
		     * install new list file
		     */

		    fflush(fo);
		    if (ferror(fo)) {
			printf("%s - error writing file\n", path1);
			remove(path1);
		    } else {
			rename(path1, path2);	/* install new list */
			printf("%s - success\n", path1);
		    }
		}
		if (fo)
		    fclose(fo);
		if (fo2)
		    fclose(fo2);
		fclose(fi);
		close(fd);	/* not really necessary I don't think */
	    }
	}
    }

    return 0;
}

/*
 * OpenAndLock() - open and lock the list file.  The locking
 * mechanism must check if the file was ripped out from under
 * us while we were blocked waiting on the lock, and reopen it
 * if so.
 */

int
OpenAndLock(const char *fileName)
{
    int fd = -1;

    for (;;) {
	if ((fd = open(fileName, O_RDWR)) >= 0) {
	    struct stat st;

	    if (flock(fd, LOCK_EX|(ForceOpt ? 0 : LOCK_NB)) < 0) {
		close(fd);
		fd = -1;
		break;
	    }
	    if (
		fstat(fd, &st) < 0 ||
		st.st_nlink == 0
	    ) {
		close(fd);
		fd = -1;
		sleep(1);
		continue;
	    }
	}
	break;
    }
    return(fd);
}

/*
 * SortList() - reverse-domain-ordered sort, case insensitive
 */

void 
SortList(FILE *fi, FILE *fo,  FILE *fobak)
{
    int n = 0;
    int i;
    char buf[4096];
    char **ary;

    while (fgets(buf, sizeof(buf), fi) != NULL) {
	fputs(buf, fobak);
	++n;
    }
    if ((ary = malloc(n * sizeof(char *))) == NULL) {
	printf("malloc() failed\n");
	exit(1);
    }
    fseek(fi, 0L, 0);
    for (i = 0; i < n; ++i) {
	if (fgets(buf, sizeof(buf), fi) == NULL)
	    break;
	if ((ary[i] = malloc(strlen(buf) + 1)) == NULL) {
	    printf("malloc() failed\n");
	    exit(1);
	}
	strcpy(ary[i], buf);
    }
    n = i;
    qsort(ary, n, sizeof(char *), emCmp);
    for (i = 0; i < n; ++i) {
	if (ary[i][0] != '-')
	    fprintf(fo, "%s", ary[i]);
	free(ary[i]);
    }
    free(ary);
}

/*
 * Compare function
 */

int
emCmp(const void *s1, const void *s2)
{
    const char *lhs = *(char **)s1;
    const char *rhs = *(char **)s2;
    const char *lp;
    const char *rp;
    int r = 0;

    /*
     * Locate end of email address
     */

    if ((lp = strchr(lhs, '>')) != NULL && lp > lhs)
	--lp;
    else if ((lp = strchr(lhs, '(')) != NULL && lp > lhs)
	--lp;
    else
	lp = lhs + strlen(lhs);

    if ((rp = strchr(rhs, '>')) != NULL && rp > rhs)
	--rp;
    else if ((rp = strchr(rhs, '(')) != NULL && rp > rhs)
	--rp;
    else
	rp = rhs + strlen(rhs);

    /*
     * Remove trailing whitespace from consideration, including a
     * potential trailing dot.
     */

    while (lp > lhs && (*lp == 0 || *lp == '\n' || *lp == ' ' || *lp == '\t' || *lp == '.'))
	--lp;
    while (rp > rhs && (*rp == 0 || *rp == '\n' || *rp == ' ' || *rp == '\t' || *rp == '.'))
	--rp;

    /*
     * Compare each subdomain until a mismatch occurs or we exhaust our
     * domain space.
     */

    do {
	while (lp > lhs && *lp != '.' && *lp != '@' && *lp != ' ' && *lp != '\t')
	    --lp;
	while (rp > rhs && *rp != '.' && *rp != '@' && *rp != ' ' && *rp != '\t')
	    --rp;
	if ((r = strcasecmp(lp, rp)) != 0)
	    break;
	if (lp > lhs)
	    --lp;
	if (rp > rhs)
	    --rp;
    } while (lp > lhs && rp > rhs);
    if (r == 0)
	r = strcasecmp(lp, rp);
    return(r);
}

