
/*
 * PROCESS.C	- PROCESS INCOMING MAIL
 */

#include "defs.h"

/************************************************************************
 *			AUTHENTICATE MESSAGE OPERATION			*
 ************************************************************************
 *
 */

Prototype void PostProc(Msg *msg);
Prototype void RequestProc(Msg *msg);
Prototype void ResponseProc(Msg *msg);
Prototype void CommandProc(Msg *msg);
Prototype void ErrorsProc(Msg *msg);

/*
 * Post to main list
 */

void
PostProc(Msg *msg)
{
    MailList *list;

    if ((list = msg->ms_List) == NULL) {
	msg->ms_ErrorFlag = 1;
	msg->ms_DeleteFlag = 1;
	msg->ms_BounceFlag |= BOUNCE_RETURN;
	msgprintf(msg, "The requested list does not exist on this server\n");
	return;
    }
    HandlePolicy(
	msg, 
	GetListVar(list, "postPolicy", 0, "AS"), 
	GetListVar(list, "postModeration", 0, "C"),
	PostExec, 
	0
    );
}

/*
 * Request for list
 */

void
RequestProc(Msg *msg)
{
    MailList *list;
    char polFlags[32];
    char modFlags[32];

    polFlags[0] = 0;
    modFlags[0] = 0;

    if ((list = msg->ms_List) == NULL) {
	msg->ms_ErrorFlag = 1;
	msg->ms_DeleteFlag = 1;
	msg->ms_BounceFlag |= BOUNCE_RETURN;
	msgprintf(msg, "The requested list does not exist on this server\n");
	return;
    }

    msg->ms_ExecScanOnly = 1;
    RequestExec(msg, 0, "", "");
    msg->ms_ExecScanOnly = 0;

    if (msg->ms_ExecScanFlags & EXSF_UNSUBSCRIBE) {
	strcat(polFlags, GetListVar(list, "unsubPolicy", 0, "-"));
	strcat(modFlags, GetListVar(list, "unsubModeration", 0, "C"));
    }
    if (msg->ms_ExecScanFlags & EXSF_SUBSCRIBE) {
	strcat(polFlags, GetListVar(list, "subPolicy", 0, "AS"));
	strcat(modFlags, GetListVar(list, "subModeration", 0, "C"));
    }
    if (msg->ms_ExecScanFlags & EXSF_GENERAL) {
	strcat(polFlags, GetListVar(list, "reqPolicy", 0, ""));
	strcat(modFlags, GetListVar(list, "reqModeration", 0, "C"));
    }
    if (msg->ms_ExecScanFlags & EXSF_MODERATOR) {
	strcat(polFlags, GetListVar(list, "modPolicy", 0, "AS"));
	strcat(modFlags, GetListVar(list, "modModeration", 0, "Y"));
    }

    HandlePolicy(
	msg, 
	polFlags, 
	modFlags,
	RequestExec, 
	0
    );
}

/*
 * Response to verification request
 *
 * Subject: line must contain ID=<id> and either ACCEPT or
 * REJECT.
 */

void
ResponseProc(Msg *msg)
{
    const char *p;
    char *subj;
    int l;
    Msg *msg2;

    msg->ms_DeleteFlag = 1;

    if ((p = findHeader(msg, "Subject:", 0, &l, NULL)) == NULL) {
	msg->ms_ErrorFlag = 1;
	msg->ms_BounceFlag |= BOUNCE_RETURN;
	msgprintf(msg, "Unable to find \"Subject\" header line.\n");
	return;
    }
    subj = dupnstr(p, l);
    if ((p = strstr(subj, "ID=")) == NULL) {
	msg->ms_ErrorFlag = 1;
	msg->ms_BounceFlag |= BOUNCE_RETURN;
	msgprintf(msg, "Unable to find ID= in subject\n");
	return;
    }
    if ((msg2 = LoadMessageById(msg, p + 3)) != NULL) {
	if (strcasestr(subj, "ACCEPT") != NULL) {
	    msg2->ms_DeleteFlag = 1;
	    if (msg2->ms_State == MSSTATE_AUTHSUBSCRIBE) {
		msg2->ms_ErrorMsg = "Message has been authenticated";
		RunMsg(msg2);
	    } else if (msg2->ms_State == MSSTATE_ACTIVE || msg2->ms_State == MSSTATE_AUTHENTICATED1) {
		msg2->ms_ErrorMsg = "Message has been authenticated";
		msg2->ms_State = MSSTATE_AUTHENTICATED1;
		lseek(msg2->ms_Fd, 1L, 0);
		write(msg2->ms_Fd, &msg2->ms_State, 1);
		RunMsg(msg2);
	    } else if (msg2->ms_State == MSSTATE_AUTHENTICATED2) {
		msg2->ms_ErrorMsg = "Message has been authenticated by moderator";
		RunMsg(msg2);
	    }
	} else if (strcasestr(subj, "REJECT") != NULL) {
	    msg2->ms_DeleteFlag = 1;
	    msg->ms_BounceFlag |= BOUNCE_RETURN;
	    msgprintf(msg,
		      "Your authentication response was rejected.\n");
	} else {
	    msg->ms_ErrorFlag = 1;
	    msg->ms_BounceFlag |= BOUNCE_RETURN;
	    msgprintf(msg,
 ("Your authentication response did not have ACCEPT or REJECT in the \"Subject\"\n"
  "line.\n")
		);
	}
	disposeMsg(msg2);
    } else {
	msg->ms_ErrorFlag = 1;
	msg->ms_BounceFlag |= BOUNCE_RETURN;
	msgprintf(msg,
 ("The message you are authenticated has already been processed or has\n"
  "timed out.\n")
	    );
    }
}

/*
 * Handle moderator commands
 */

void
CommandProc(Msg *msg)
{
    char polFlags[32];
    char modFlags[32];

    polFlags[0] = 0;
    modFlags[0] = 0;

#ifdef NOTDEF
    msg->ms_ExecScanOnly = 1;
    RequestExec(msg, 1);
    msg->ms_ExecScanOnly = 0;

    if (msg->ms_ExecScanFlags & EXSF_UNSUBSCRIBE) {
	strcat(polFlags, GetListVar(list, "unsubPolicy", 0, ""));
	strcat(modFlags, GetListVar(list, "unsubModeration", 0, ""));
    }
    if (msg->ms_ExecScanFlags & EXSF_SUBSCRIBE) {
	strcat(polFlags, GetListVar(list, "subPolicy", 0, ""));
	strcat(modFlags, GetListVar(list, "subModeration", 0, ""));
    }
    if (msg->ms_ExecScanFlags & EXSF_GENERAL) {
	strcat(polFlags, GetListVar(list, "reqPolicy", 0, ""));
	strcat(modFlags, GetListVar(list, "reqModeration", 0, ""));
    }
    if (msg->ms_ExecScanFlags & EXSF_MODERATOR) {
	strcat(polFlags, GetListVar(list, "modPolicy", 0, ""));
	strcat(modFlags, GetListVar(list, "modModeration", 0, ""));
    }
#endif

    HandlePolicy(msg, "AS", "C", RequestExec, 1);
}

/*
 * Handle errors
 */

void 
ErrorsProc(Msg *msg)
{
    msg->ms_DeleteFlag = 1;
    msg->ms_BounceFlag |= BOUNCE_CCERRORS;
}

