
/*
 * QUEUE.C
 *
 * Run queue - actually, for now, scan all the mailing lists and look for
 * digests that need to be run.
 */

#include "defs.h"

Prototype void RunQueue(const char *listName);

void RunQueueFile(const char *listName);

void 
RunQueue(const char *listName)
{
    DIR *dir;
    struct dirent *den;

    if (chdir(BaseDir) < 0)
	fatal(errno, "chdir(%s)", BaseDir);
    if (chdir("lists") < 0)
	fatal(errno, "chdir(%s/spool)", BaseDir);

    if (listName) {
	RunQueueFile(listName);
    } else {
	if ((dir = opendir(".")) != NULL) {
	    while ((den = readdir(dir)) != NULL) {
		if (den->d_name[0] == '.')
		    continue;
		RunQueueFile(den->d_name);
	    }
	    closedir(dir);
	}
    }
}

/*
 * Run a queue file.  Note that we must GetMailList() in order
 * to obtain the master lock on the config file to prevent race
 * conditions.
 */

void
RunQueueFile(const char *listName)
{
    MailStats *mst;
    MailList *list = GetMailList(listName, 0);

    printf("list %-25s ", listName);
    fflush(stdout);

    if ((mst = ReadMailStats(listName)) != NULL && mst->st_DigestNext) {
	long dt = (long)time(NULL) - mst->st_DigestNext;
	long adt = (dt < 0) ? -dt : dt;

	if (mst->st_DigestMsgs) {
	    if (dt >= 0)
		printf("past due - running\n");
	    else
		printf("run in %02d:%02d:%02d\n",
		    (int)(adt / (60 * 60)),
		    (int)(adt / 60 % 60),
		    (int)(adt % 60)
		);
	} else {
	    printf("digest empty\n");
	}

	if (dt >= 0) {
	    FILE *fi;

	    if ((fi = OpenListFile(listName, "digest.tmp", 0, -1)) != NULL) {
		fclose(fi);

		DoRunDigest(list, NULL);
	    }
	}
    } else {
	printf("digests off\n");
    }
    if (mst)
	WriteMailStats(mst);
    if (list)
	CloseMailList(list);
}

